#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const xlsx_1 = __importDefault(require("xlsx"));
const fast_glob_1 = require("fast-glob");
// 配置路径
const SHEET_NAME = 'translate';
const SRC_DIR = path_1.default.join(__dirname, '../../src');
const TS_PATH = path_1.default.join(SRC_DIR, '**/*.ts');
const VUE_PATH = path_1.default.join(SRC_DIR, '**/*.vue');
const XLSX_PATH = path_1.default.join(SRC_DIR, 'xls/language.xlsx');
// 匹配正则表达式
const VUE_PATTERN = /\$t\(['"](.+?)['"]\)/g;
const TS_PATTERN = /translate\(['"](.+?)['"]\)/g;
// 提取所有唯一key
function extractKeys() {
    return __awaiter(this, void 0, void 0, function* () {
        const files = yield (0, fast_glob_1.async)([
            TS_PATH.replace(/\\/g, '/'),
            VUE_PATH.replace(/\\/g, '/')
        ]);
        const keys = new Set();
        for (const file of files) {
            const content = fs_1.default.readFileSync(file, 'utf8');
            const patterns = file.endsWith('.vue') ? VUE_PATTERN : TS_PATTERN;
            let match;
            while ((match = patterns.exec(content)) !== null) {
                keys.add(match[1]);
            }
        }
        return Array.from(keys).sort();
    });
}
// 更新xlsx文件
function updateXlsx(keys) {
    let workbook;
    let worksheet;
    const types = ['str', 'str', 'str', 'str', 'str', 'str'];
    const headers = ['$LanguageCfg', 'desc_id', 'zh_CN', 'en_US', 'fr_FR', 'ja_JP', 'zh_HK'];
    try {
        workbook = xlsx_1.default.readFile(XLSX_PATH);
        worksheet = workbook.Sheets[SHEET_NAME];
    }
    catch (e) {
        // 文件不存在时创建新workbook
        workbook = xlsx_1.default.utils.book_new();
        worksheet = xlsx_1.default.utils.aoa_to_sheet([headers, types]);
        xlsx_1.default.utils.book_append_sheet(workbook, worksheet, SHEET_NAME);
    }
    // 获取现有数据
    const jsonData = xlsx_1.default.utils
        .sheet_to_json(worksheet, { header: 1 })
        .filter((v) => v.length > 0)
        .map((v, i) => {
        return i !== 0 ? new Array(1).concat(v.filter((vv) => !!vv)) : v.filter((vv) => !!vv);
    });
    //现有key    
    const existingKeys = new Set(jsonData.slice(2).map(row => row[1]));
    // 添加新key
    keys.forEach(key => {
        if (!existingKeys.has(key)) {
            const arr = new Array(headers.length);
            arr[1] = key;
            arr[2] = key;
            jsonData.push(arr);
        }
    });
    // 更新worksheet并保存
    const newWorksheet = xlsx_1.default.utils.aoa_to_sheet(jsonData, { origin: 'B2' });
    //添加批注
    newWorksheet['C2'].c = [{ t: '#[primary=true]' }];
    newWorksheet['B2'].c = [{ t: '#[mainTable=game_translate/xls/language/base.LanguageCfg]' }];
    //保存配置表
    workbook.Sheets[SHEET_NAME] = newWorksheet;
    xlsx_1.default.writeFile(workbook, XLSX_PATH);
}
// 主流程
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        const keys = yield extractKeys();
        updateXlsx(keys);
        console.log(`更新完成，共处理${keys.length}个国际化字段`);
    });
}
main().catch(console.error);
//# sourceMappingURL=i18n.js.map