import path from 'path';
import fs, { Stats } from 'fs';
import { callbackToPromise } from './util'

/**
 * 安全删除目录或文件
 * @param _path 要删除的文件或目录路径
 * @throws {Error} 当文件操作失败时抛出异常
 */
export async function removeDirOrFile(_path: string) {
    if (!fs.existsSync(_path)) return;
    const file = await callbackToPromise(fs.stat)(_path) as Stats;
    if (file.isFile()) {
        await callbackToPromise(fs.rm)(_path);
    } else if (file.isDirectory()) {
        const fileDir = await callbackToPromise(fs.readdir)(_path) as string[];
        for (let i = 0; i < fileDir.length; i++) {
            await removeDirOrFile(path.join(_path, fileDir[i]))
        }
        await callbackToPromise(fs.rmdir)(_path);
    }
}
