#!/usr/bin/env node
import fs from 'fs';
import path from 'path';
import XLSX from 'xlsx';
import { async as glob } from 'fast-glob';

// 配置路径
const SHEET_NAME = 'translate';
const SRC_DIR = path.join(__dirname, '../../src');
const TS_PATH = path.join(SRC_DIR, '**/*.ts');
const VUE_PATH = path.join(SRC_DIR, '**/*.vue');
const XLSX_PATH = path.join(SRC_DIR, 'xls/language.xlsx');

// 匹配正则表达式
const VUE_PATTERN = /\$t\(['"](.+?)['"]\)/g;
const TS_PATTERN = /translate\(['"](.+?)['"]\)/g;

// 提取所有唯一key
async function extractKeys() {
    const files = await glob([
        TS_PATH.replace(/\\/g, '/'),
        VUE_PATH.replace(/\\/g, '/')
    ]);
    const keys: Set<string> = new Set();
    for (const file of files) {
        const content = fs.readFileSync(file, 'utf8');
        const patterns = file.endsWith('.vue') ? VUE_PATTERN : TS_PATTERN;
        let match;
        while ((match = patterns.exec(content)) !== null) {
            keys.add(match[1]);
        }
    }
    return Array.from(keys).sort();
}

// 更新xlsx文件
function updateXlsx(keys: string[]) {
    let workbook;
    let worksheet;
    const types = ['str', 'str', 'str', 'str', 'str', 'str'];
    const headers = ['$LanguageCfg', 'desc_id', 'zh_CN', 'en_US', 'fr_FR', 'ja_JP', 'zh_HK'];
    try {
        workbook = XLSX.readFile(XLSX_PATH);
        worksheet = workbook.Sheets[SHEET_NAME];
    } catch (e) {
        // 文件不存在时创建新workbook
        workbook = XLSX.utils.book_new();
        worksheet = XLSX.utils.aoa_to_sheet([headers, types]);
        XLSX.utils.book_append_sheet(workbook, worksheet, SHEET_NAME);
    }

    // 获取现有数据
    const jsonData = XLSX.utils
        .sheet_to_json(worksheet, { header: 1 })
        .filter((v: any) => v.length > 0)
        .map((v: any, i: number) => {
            return i !== 0 ? new Array(1).concat(v.filter((vv: any) => !!vv)) : v.filter((vv: any) => !!vv)
        });
    //现有key    
    const existingKeys = new Set(jsonData.slice(2).map(row => row[1]));
    // 添加新key
    keys.forEach(key => {
        if (!existingKeys.has(key)) {
            const arr = new Array(headers.length)
            arr[1] = key;
            arr[2] = key;
            jsonData.push(arr);
        }
    });
    // 更新worksheet并保存
    const newWorksheet = XLSX.utils.aoa_to_sheet(jsonData, { origin: 'B2' } as any);
    //添加批注
    newWorksheet['C2'].c = [{ t: '#[primary=true]' }];
    newWorksheet['B2'].c = [{ t: '#[mainTable=game_translate/xls/language/base.LanguageCfg]' }];
    //保存配置表
    workbook.Sheets[SHEET_NAME] = newWorksheet;
    XLSX.writeFile(workbook, XLSX_PATH);
}

// 主流程
async function main() {
    const keys = await extractKeys();
    updateXlsx(keys);
    console.log(`更新完成，共处理${keys.length}个国际化字段`);
}

main().catch(console.error);